.\" $Id: libstr.man 13 2004-03-27 20:21:34Z amyzing $
.TH libstr 3 6/22/97 "strdb 0.5.0" "Strings DB Developer's Manual"

.SH NAME

libstr \- The Strings DB library

.SH SYNOPSIS

.SS Index file functions
.B #include <strdb/strdb.h>

.SS Sorting functions
.B #include <strdb/sortdb.h>

.SS Key file functions
.B #include <strdb/keydb.h>

.SS Error functions
.B #include <strdb/errdb.h>

.SS Maintenance functions
.B #include <strdb/maintdb.h>

.SH DESCRIPTION

The 
.B strdb 
library may be used in the development of other applications.  To do so, it
is only necessary to link in the library (by default, not a shared library),
and to include the appropriate header files.  Included library functions
operate on and verify index and key files, retrieve records and keysets, add
and delete records, and generate certain library-wide errors.
.PP
The most general functions are used for retrieving a given record and
storing it into memory.  The library also defines the structures that define
a record, and certain constants used in manipulation of records, keys, and
files.

.SS Types
Note: most structures are also typedefs.  See 
.BR Structures .
.PP
.B #include <strdb/strdb.h>
.PP
.B typedef unsigned short sdb_record_t
.PP
.B sdb_record_t
defines the type of integer used to store the pointer to an offset into a
record's data element.  Thus, it also defines (implicitly, but explicitly in
a constant, see below) the maximum size of a record's data element.  In most
current environments, this sets maximum record data size to 64K.

.SS Constants
.B #include <strdb/strdb.h>
.PP
.B const char sdb_VMAJOR
.br
.B const char sdb_VMINOR
.br
.B const char sdb_VPATCH
.br
.B const char sdb_DEFSC
.br
.B const char sdb_MAGIC[4]
.br
.B const unsigned short MAXLINELEN
.br
.B const sdb_RECSIZE sdb_RECORDMAX
.br
.B const unsigned char sdb_SORT_CASE
.br
.B const unsigned char sdb_SORT_REVERSE
.br
.B const unsigned char sdb_SORT_IWHITE
.br
.B const unsigned char sdb_SORT_ALNUM
.PP
The three version constants are stored into index and key files, and are
to be used by applications to verify that the record structure is one that
they can read.  In release versions,
.RB ( sdb_VMAJOR " >= 1), "
the major version number changes only when data structures change.  Minor
version number changes on any change to the library functions or constants;
it should be safe to ignore these changes when reading an existing index or
key file and their associated data files.  Patch level changes are
considered insignificant.  In pre-release versions, 
.RB ( sdb_VMAJOR " = 0 ), "
the minor version number is equivalent to release versions' major version
number (that is, the minor version number changes when there is a change to
structures, and all other changes are reflected in the patch level).
.PP
.B sdb_DEFSC 
defines the default signal character 
.RB ( % ), 
the character used to signal a field indicator or record terminator in a
data file.  This is only the 
.I default
, please note.  Likewise, 
.B sdb_MAGIC 
defines a string embedded into key and index files which may be used to
verify that the files are, indeed, key and index files (a function is
provided for this purpose).
.PP
The two constants
.BR MAXLINELEN " and " sdb_RECORDMAX 
both define upper limits, the first for line length in data files (this
limitation should be removed, at some point, if possible), and the second
for record data 
.IR "in toto" .  
This second constant simply makes explicit the implicit limitation defined
by the 
.B sdb_record_t 
data type.
.PP
The four sort flags are defined in 
.I strdb.h 
because they are stored in the created index file.
.B sdb_SORT_CASE 
(0x01) is set if the sort is case-sensitive, 
.B sdb_SORT_REVERSE 
(0x02) is set if the sort is in reverse order, 
.B sdb_SORT_IWHITE 
(0x04) is set if the sort ignores whitespace (and control) characters, 
.B sdb_SORT_ALNUM 
(0x08) is set if the sort is on alphanumeric characters only.  Four more
library-wide flags are available, but undefined, and there are eight
application-specific flags stored into each index file as well.
.PP
.B #include <strdb/errdb.h>
.PP
.BR const char *sdb_err_string[]
.br
.BR "#define sdb_ENOTIDX " 1
.br
.BR "#define sdb_ENOTKEY " 2
.br
.BR "#define sdb_EIDXDATE " 3
.br
.BR "#define sdb_EKEYDATE " 4
.br
.BR "#define sdb_EIDXRANGE " 5
.br
.BR "#define sdb_EUSAGE " 6
.br
.BR "#define sdb_EVER " 7
.br
.BR "#define sdb_EFREAD " 8
.br
.BR "#define sdb_EFWRITE " 9
.br
.BR "#define sdb_ENOENT " 10
.br
.BR "#define sdb_EMEMFAULT " 12
.br
.BR "#define sdb_ECHAOS " 16
.PP
This string constant is intended to be indexed by the #define'd constants to
generate a user-friendly (well, relatively ... this is unix, after all)
message when an application, core utility, or library function blows up.
The first two are returned when the magic is not found in a key or index
file; the second two when the timestamp of the referenced file (index file
for a key file, data file for an index file) is later than the stored
timestamp of the referenced file; a range error is returned if an index
number is greater than the total number of stored pointers in an index file. 
The usage error code is to be returned when an application or core utility
is called with unknown or incorrect parameters; the version code when the
major version number of the program is not identical to the major version
number stored in the index or key file.  There are three file errors, to be
returned on a failed read, write, or attempt to access or create a file. 
The memory fault error code is to be returned as a graceful alternative to a
segfault, when 
.BR malloc (3) 
fails.  The final error is a catchall, to be returned whenever things are so
messed up that there must be some severe logical error in the application,
core utilities, or library.  "Programmer's fault," so to speak.
.PP
.B #include <strdb/maintdb.h>
.PP
.B const char *sdb_maint_err[]
.br
.BR "#define sdb_EMNODATA " 1
.br
.BR "#define sdb_EMEARLYTERM " 2
.br
.BR "#define sdb_EMNOTERM " 3
.br
.BR "#define sdb_EMLONGLINE " 4
.br
.BR "#define sdb_EMINTERNTERM " 5
.br
.BR "#define sdb_EMFORMAT " 6
.br
.BR "#define sdb_EMUNKNOWN " 7
.PP
.I maintdb.h 
defines another set of error codes and messages, with the same basic
structure as the main set of error codes and messages.  Note, however, that
the return value from a program that fails using these error codes will be
the error codes defined here, plus 16 (this avoids any possibility of
confusion with the main error codes, which range from one to sixteen).  All
of these are errors returned by functions which attempt to parse a single
record for consistency.  Tests can return terminator errors: starting with a
terminator, failing to end with a terminator, or attempting to contain a
terminator, and the associated error that there is no data at all.  Lines
must be of a certain length or less.  If a format for the record is
specified (only certain fields may be included, perhaps in a certain order),
then the format error may be returned when an inappropriate field is
encountered.  The unknown error is a graceful way of saying that the
programmer messed up.

.SS Structures

sdb_header
.PP
sdb_recptr
.PP
linerr
.PP
sdb_keyhead
.PP
sdb_key

.SS Functions

.SH FILES

.I /usr/local/lib/libstr.[a|so]
.br
.I /usr/local/include/strdb/errdb.h
.br
.I /usr/local/include/strdb/keydb.h
.br
.I /usr/local/include/strdb/maintdb.h
.br
.I /usr/local/include/strdb/sortdb.h
.br
.I /usr/local/include/strdb/strdb.h

.SH DIAGNOSTICS

In general, applications based on the 
.B strdb 
library should return errors defined in 
.IR errdb.h .
.PP
.I errdb.h 
also defines an array of pointers to strings which may be displayed to the
user when a fault is encountered.

.SH BUGS

None known.

.PP
Heh.

.SH AUTHOR

.RB "Amelia A. Lewis \- " strdb@talsever.com
