/*
 * strdb.h
 *
 * Header for the strings db toolkit.
 *
 * This file defines standard structures and prototypes for use by other
 * programs in the package.  For details of implementation for the functions,
 * see the comments in strdb.c for each function.
 *
 * version 0.2.0, appears to be working again.  September 22, 1996.
 * 0.3.0, March 23, 1997.  Keys.  (moved to keydb)
 * 0.4.0, March 24, 1997 (I'm a busy girl this weekend).  Store sizes.
 *
 * Copyright (c) 1996 Amelia A. Lewis
 * Distributed under GPL.  See COPYING.
 *
 * $Id: strdb.h 13 2004-03-27 20:21:34Z amyzing $
 */

#ifndef sdb_STRDB_H
#define sdb_STRDB_H

#include <sys/param.h>
#include <sys/types.h>

typedef unsigned short sdb_record_t;	/* type used for storing size of records */
					/* Max 64K, as a rule */

extern const char sdb_VMAJOR;		/* major version number */
extern const char sdb_VMINOR;		/* minor version number */
extern const char sdb_VPATCH;		/* patch number */
					/* Note: if major < 1, no guarantees
					   of future compatibility are made */
extern const char sdb_DEFSC;		/* default signal character, '%' */
extern const char sdb_MAGIC[4];		/* magic for identifying indices, "aal" */
extern const unsigned short MAXLINELEN;	/* has to be something, is 1K OK? */
extern const sdb_record_t sdb_RECORDMAX; /* USHRT_MAX, usually 64K */

extern const unsigned char sdb_SORT_CASE;
extern const unsigned char sdb_SORT_REVERSE;
extern const unsigned char sdb_SORT_IWHITE;
extern const unsigned char sdb_SORT_ALNUM;
extern const unsigned char sdb_SORT_NUMBER;

/*
 * sdb_header:
 *
 * This is the structure stored as the header into each index file, usually
 * by the utility idxstr, and read by a variety of utilities for their own
 * nefarious purposes.  The values stored into many elements are those of
 * the corresponding constants above as compiled into idxstr.  Most are
 * there to prevent kludgification, when version numbers mismatch, and so on.
 * There is currently no checking for that.
 */
typedef struct
{
    char sdb_major;			/* sdb_VMAJOR */
    char sdb_minor;			/* sdb_VMINOR */
    char sdb_patch;			/* sdb_VPATCH */
    char sdb_sigch;			/* sdb_DEFSC, or other chosen */
    char sdb_sorton;			/* field sorted on, if any */
    char sdb_sortflags;			/* flags used when sorting */
    char sdb_appflags;			/* flags set by application */
    char sdb_reserved_nonsense;		/* to make it come out even */
    char sdb_magic[4];			/* sdb_MAGIC */
    unsigned long sdb_total;		/* total number of pointers stored */
    time_t sdb_mtime;			/* mod time for indexed file */
    char sdb_filename[MAXPATHLEN];	/* absolute path name of indexed file */
} sdb_header;

/*
 * sdb_recptr:
 *
 * This structure is used chiefly by indexing utilities, permitting creation
 * of a linked list of pointers into the file, each with a next pointer.
 * Note that the next pointer is defined as void*; cast it.
 */
typedef struct
{
    void *next;
    off_t data_os;
    sdb_record_t data_size;
    char *data;
} sdb_recptr;
              
/*
 * The remainder of the file is function prototypes.  For explanations, see
 * the implementations.
 */

sdb_recptr *
sdb_pointer(FILE *, unsigned long);

sdb_recptr *
sdb_nextptr(FILE *);

sdb_recptr *
sdb_prevptr(FILE *);

char
sdb_delim(char *, char);

char *
sdb_ndinr(char *, char);

char *
sdb_finr(char *, char, char);

int
sdb_unicorn(FILE *, char);

FILE *
sdb_findindex(char *, int *, int, int);

int
sdb_recalloc(FILE *, sdb_recptr *);

int
sdb_stripterm(sdb_recptr *, char);

void
sdb_recfree(sdb_recptr *);

unsigned long
sdb_grep(FILE *, FILE *, char *, char, off_t *, unsigned long, int, int, int, int, int);

void
sdb_version(char *, char *);

#endif /* sdb_STRDB_H */

